<?php

namespace Drupal\Tests\widget_ingestion\Unit;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Queue\QueueWorkerInterface;
use Drupal\Core\Queue\QueueWorkerManagerInterface;
use Drupal\Tests\UnitTestCase;
use Drupal\widget_ingestion\WidgetRemoteRepository;
use GuzzleHttp\Client;
use GuzzleHttp\Psr7\Response;
use Prophecy\Argument;

/**
 * Unit tests for the WidgetRemoteRepository.
 *
 * @group widget_ingestion
 *
 * @coversDefaultClass \Drupal\widget_ingestion\WidgetRemoteRepository
 * @SuppressWarnings(PHPMD.StaticAccess)
 */
final class WidgetRemoteRepositoryTest extends UnitTestCase {

  /**
   * The system under test.
   *
   * @var \Drupal\widget_ingestion\WidgetRemoteRepository
   */
  private $theSut;

  /**
   * The queue worker.
   *
   * @var \Drupal\Core\Queue\QueueWorkerInterface
   */
  private $queueWorker;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();
    $config_factory = $this->prophesize(ConfigFactoryInterface::class);
    $config = $this->prophesize(ImmutableConfig::class);
    $config->get('endpoint')->willReturn('foo-bar');
    $config_factory->get('widget_ingestion.settings')
      ->willReturn($config->reveal());
    $client = $this->prophesize(Client::class);
    $options = [
      'headers' => [
        'Accept' => 'application/json',
      ],
    ];
    $response = new Response(200, [], '[{"lorem": "ipsum","shortcode":"foo"}]');
    $this->queueWorker = $this->prophesize(QueueWorkerInterface::class);
    $this->queueWorker->processItem(Argument::type('array'))->willReturn();
    $queue_worker_manager = $this->prophesize(QueueWorkerManagerInterface::class);
    $queue_worker_manager->createInstance(Argument::type('string'))
      ->shouldBeCalledTimes(1)
      ->willReturn($this->queueWorker->reveal());
    $client->get('foo-bar', $options)->willReturn($response);
    $file_system = $this->prophesize(FileSystemInterface::class);
    $this->theSut = new WidgetRemoteRepository(
      $client->reveal(),
      $config_factory->reveal(),
      $queue_worker_manager->reveal(),
      $file_system->reveal()
    );
  }

  /**
   * @covers ::listAll
   */
  public function testListAll() {
    self::assertEquals([
      ['lorem' => 'ipsum', 'shortcode' => 'foo'],
    ], $this->theSut->listAll());
  }

  /**
   * @covers ::ingestAll
   */
  public function testIngestAll() {
    $this->queueWorker
      ->processItem(Argument::type('array'))
      ->shouldBeCalledTimes(1);
    self::assertEquals(
      [
        ['lorem' => 'ipsum', 'shortcode' => 'foo', '_force_ingestion' => FALSE],
      ],
      $this->theSut->ingestAll()
    );
  }

  /**
   * @covers ::ingestWidget
   */
  public function testIngestWidget() {
    $this->queueWorker
      ->processItem(Argument::type('array'))
      ->shouldBeCalledTimes(1);
    self::assertEquals(
      ['lorem' => 'ipsum', 'shortcode' => 'foo', '_force_ingestion' => TRUE],
      $this->theSut->ingestWidget('foo', TRUE)
    );
  }

}
